/* 

https://github.com/wycats/handlebars.js ????

use by : http://jvitela.github.io/mustache-wax/ 

Mustache.render(
        "{{ name | uppercase }}, {{ dob | date }}, {{ ssnum | lpad : 10 : '0' }}", 
        {
            name:   "john doe",
            dob:    new Date(1981, 2, 24),
            ssnum:  12345
        }
    );
	
	
	Formatters can be applied to expressions in templates using the following syntax:

	{{ expression | formatter }}
	
	
	Formatters can be applied to the result of another formatter. This is called "chaining" and uses the following syntax:

	{{ expression | formatter1 | formatter2 | ... }}
	
	
	Formatters may have arguments. The syntax for this is

	{{ expression | formatter : argument1 : argument2 : ... }}
	
	Arguments can be either an integer, a real, a string or another expression. 
	Integer arguments example:
	
	
	For example, assuming we have the following formatter and data:

    Mustache.Formatters = {
        "add": function (one, two) {
            return one + two;
        }
    }

    var data = {
        ten:    10,
        twenty: 20
    };
	We could use the following templates

		{{ ten | add: 5}}
		{{ twenty | add: 5.25 | add:-.25 }}
		{{ ten | add : 3.14159 | add : twenty | add:-3}}
	The result will be:

		15
		25
		30.14159
	
	


*/

if (!String.prototype.trim) {
  String.prototype.trim = function () {
    return this.replace(/^[\s\uFEFF\xA0]+|[\s\uFEFF\xA0]+$/g, '');
  };
}

Mustache.Formatters = {
        "getuniqueid": function (str) {
            return viwutils.getuniqueID();
        },
		"uppercase": function (str) {
            return str.toUpperCase();
        },
		"numtoopacity": function (n) {
            return n/100;
        },
        "lpad": function (str, num, sep) {
            sep = sep || " ";
            str = "" + str;
            var filler = "";
            while ((filler.length + str.length) < num) { filler += sep };
            return (filler + str).slice(-num);
        },
        "date": function (dt) {
            var lpad  = Mustache.Formatters.lpad, 
                day   = lpad(dt.getDate(), 2, "0"),
                month = lpad(dt.getMonth()+1, 2, "0");
            return  day + "/" + month + "/" + dt.getFullYear();
        },
		"urlencode": function (str) {
            return viwutils.urlencode(str);
        },
		"replaceasctoquote": function (str) {
            return viwutils.replaceasctoquote(str);
        },
		"explode": function (str, delimiter) {
			return viwutils.explode(delimiter, str);
		},
		// pricingtable
		"atolidiv": function (a, cl) {
			var s= "";
			for (var i=0; i<a.length;i++){
				s+='<li><div class="'+cl+'">'+a[i]+'</div></li>'
			}
			return s;
		},
		
		// pricingtable
		"atostringcoma": function (a) {
			var s= "";
			for (var i=0; i<a.length;i++){
				s+=a[i]+', ';
			}
			s = s.substring(0, s.length-2);
			return s;
		},
		
		"sccontenttolabelcoma": function (mycontent, title){
			var a = viwutils.parseShorcodeWidget(mycontent, 100);
			var s = '';
			for (var i=0; i<a.length ; i++){
				var item = a[i];
				s += item[title]+', ';
			}
			s = s.substring(0, s.length-2);
			return s;
		},
		
		"sccontenttoicon": function (mycontent, title){
			var a = viwutils.parseShorcodeWidget(mycontent, 100);
			var s = '';
			for (var i=0; i<a.length ; i++){
				var item = a[i];
				s += '<i class="'+item[title]+'" ></i> ';
			}
			s = s.substring(0, s.length-2);
			return s;
		},
		
		"icon_alignement": function (str) {
			switch (str) {
				case "icon_middleleft":
					return ' vwi_flexAlignItemsCenter vwi_flexJustifyStart';
				break;
				case "icon_middleright":
					return ' vwi_flexAlignItemsCenter vwi_flexJustifyEnd';
				break;
				case "icon_middlecenter":
					return ' vwi_flexAlignItemsCenter vwi_flexJustifyCenter';
				break;
			}
           
        },
		
		// divider fetpen
		"item_alignement": function (str) {
			switch (str) {
				case "topleft":
					return ' vwi_flexAlignItemsStart vwi_flexJustifyStart';
				break;
				case "topright":
					return ' vwi_flexAlignItemsStart vwi_flexJustifyEnd';
				break;
				case "topcenter":
					return ' vwi_flexAlignItemsStart vwi_flexJustifyCenter';
				break;
				case "middleleft":
				case "left":
					return ' vwi_flexAlignItemsCenter vwi_flexJustifyStart';
				break;
				case "middleright":
				case "right":
					return ' vwi_flexAlignItemsCenter vwi_flexJustifyEnd';
				break;
				case "middlecenter":
				case "center":
					return ' vwi_flexAlignItemsCenter vwi_flexJustifyCenter';
				break;
				case "bottomleft":
					return ' vwi_flexAlignItemsEnd vwi_flexJustifyStart';
				case "bottomright":
					return ' vwi_flexAlignItemsEnd vwi_flexJustifyEnd';
				break;
				case "bottomcenter":
					return ' vwi_flexAlignItemsEnd vwi_flexJustifyCenter';
				break;
			}
           
        },
		
		// blockote
		"border_color": function (str) {
			switch (str) {
				case "default":
					return 'border-left-color';
				break;
				case "reverse":
					return 'border-right-color';	
				break;
				case "bordered":
					return 'border-color';
				break;
			}
		},
		
		// image
		
		"imgpath": function (image) {
			return VIWPROP.root_basepath_image+image;
		},
		
		// advanced margin   
		"adv_margin": function (adv_mc, adv_mun, adv_mt, adv_mb, adv_ml, adv_mr) {
			var margin_unit = "px";
				if (adv_mun!="") margin_unit = adv_mun;
			var style='';
			if (adv_mc=="1") {
				if (viwutils.intval(adv_mt)>0 )
					style += ' margin-top:'+adv_mt+margin_unit+'; ';
				if (viwutils.intval(adv_mb)>0 )
					style += ' margin-bottom:'+adv_mb+margin_unit+'; ';
				if (viwutils.intval(adv_mr)>0 )
					style += ' margin-right:'+adv_mr+margin_unit+'; ';
				if (viwutils.intval(adv_ml)>0 )
					style += ' margin-left:'+adv_ml+margin_unit+'; ';
			}
			
			return style;
		},
		
		
		
		
		
		
		
    };

/*
var tmptest = {
        "explode2": function (str, delimiter) {
			return viwutils.explode(delimiter, str);
		},
}	;	

Mustache.Formatters = Object.assign(Mustache.Formatters, tmptest);
console.dir(Mustache.Formatters);
*/
