<?php
/**
 * Classe plxForm
 *
 * @package PLX
 * @author	Evrard J.
 **/
class spxForm {

	/*
	 * Méthode générique qui traite les propriétés communes aux champs d'un formulaire
	 * @param   array(id, name, onclick, disabled, style, tabindex)
	 * @return  string
	 * @author	Evrard J.
	 */
	public static function base($params = array()) {
		$s = '';
		if(isset($params['id']))
			$s .= " id=\"{$params['id']}\"";
		else
			$s .= isset($params['name']) 	? " id=\"id_{$params['name']}\"" 			: " id=\"id_".rand(1000,9999)."\"";
		$s .= isset($params['name'])		? " name=\"{$params['name']}\""				: '';
		$s .= isset($params['onclick'])		? " onclick=\"{$params['onclick']}\""		: '';
		$s .= isset($params['disabled'])	? " disabled=\"{$params['disabled']}\""		: '';
		$s .= isset($params['style'])		? " style=\"{$params['style']}\""			: '';
		$s .= isset($params['tabindex'])	? " tabindex=\"{$params['tabindex']}\""		: '';
		$s .= isset($params['innerextra'])	? " \"{$params['innerextra']}\""		: '';
		$s .= isset($params['readonly'])	? " readonly"		: '';
		return $s;
	}

	/*
	 * Méthode qui retourne un label
	 * Exemple d'appel
	 *		echo plxForm::label(array('for'=>'id_username', 'label'=>'Nom utilisateur :'));
	 * @param   array(for, label, style, tabindex, class, extra)
	 * @return  string
	 * @author	Evrard J.
	 */
	public static function label($params = array()) {
		$style = isset($params['style'])	? " style=\"{$params['style']}\""					: '';
		$class = isset($params['class'])		? " class=\"form-elmnt label {$params['class']}\""	: ' class="form-elmnt label"';
		$s  = "\n<label for=\"{$params['for']}\"{$style}{$class}>{$params['label']}</label>";
		$s .= isset($params['extra']) ? $params['extra'] : '';
		return $s;
	}

	/*
	 * Méthode qui retourne un champ input de type text
	 * @param   array(id, name, class, onclick, disabled, style, tabindex, class, size, value, maxlength, placeholder, extra)
	 * @return  string
	 * @author	Evrard J.
	 */
	public static function text($params = array()) {
		$s  = "\n<input type=\"text\"";
		$s .= self::base($params);
		$s .= isset($params['class'])		? " class=\"form-elmnt text {$params['class']}\""	: ' class="form-elmnt text"';
		$s .= isset($params['size'])		? " size=\"{$params['size']}\""					: '';
		$s .= isset($params['value'])		? " value=\"{$params['value']}\""					: '';
		$s .= isset($params['maxlength'])	? " maxlength=\"{$params['maxlength']}\""			: '';
		$s .= isset($params['placeholder'])	? " placeholder=\"{$params['placeholder']}\""		: '';
		$s .= " />";
		$s .= isset($params['extra']) ? $params['extra'] : '';
		return $s;
	}

	/*
	 * Méthode qui retourne une zone de saisie de type textarea
	 * @param   array(id, name, class, onclick, disabled, style, tabindex, class, cols, rows, value, extra)
	 * Exemple d'appel
	 *		echo plxForm::textarea(array('name' => 'content', 'value'=>'Lorem Ipsum...', 'cols'=>30, 'rows'=>10));
	 * @return  string
	 * @author	Evrard J.
	 */
	public static function textarea($params = array()) {
		$s  = "\n<textarea";
		$s .= self::base($params);
		$s .= isset($params['class'])		? " class=\"form-elmnt textarea {$params['class']}\""	: ' class="form-elmnt textarea"';
		$s .= isset($params['cols'])		? " cols=\"{$params['columns']}\""						: '';
		$s .= isset($params['rows'])		? " rows=\"{$params['rows']}\""							: '';
		$s .= '>';
		$s .= isset($params['value'])		? $params['value']										: '';
		$s .= "</textarea>";
		$s .= isset($params['extra']) ? $params['extra'] : '';
		return $s;
	}

	/*
	 * Méthode qui retourne une liste déroulante
	 * Exemple d'appel:
	 * 		echo plxForm::select(
	 *			array(
	 *				'name'	=> 'profil[]',
	 *				'selected' => 'profil_2',
	 *				'id'	=> 'id_profil',
	 *				'prompt'=> 'Pour la sélection',
	 *				'data' => array(
	 *					'profil_1' => 'Administrateur',
	 *					'profil_2' => 'Modérateur'
	 *				)
	 *			)
	 *		);
	 * @param   array(id, name, class, onclick, disabled, style, tabindex, class, prompt, selected, data => array( 'value' => 'text', ...), extra)
	 * @return  string
	 * @author	Evrard J.
	 */
	public static function select($params = array()) {
		$s  = "\n<select";
		$s .= self::base($params);
		$s .= isset($params['class'])	? " class=\"form-elmnt select {$params['class']}\""		: ' class="form-elmnt select"';
		$s .= '>';
		$s .= isset($params['prompt'])	? "\n<option value=\"\">{$params['prompt']}</option>" : '';
		if(isset($params['data']) AND is_array($params['data'])) {
			foreach($params['data'] as $k => $v) {
				//if(isset ($params['option_class'])){
					//$class =
				//}
				if(isset($params['selected']) AND $params['selected']==$k) {
					$s .= "\n<option value=\"{$k}\" selected=\"selected\">{$v}</option>";
				} else {
					$s .= "\n<option value=\"{$k}\">{$v}</option>";
				}
			}
		}
		$s .= "\n</select>";
		$s .= isset($params['extra']) ? $params['extra'] : '';
		return $s;
	}

	# add dataclass to skin option and add datastyle to skin option
	public static function select2($params = array()) {
		$placeholder="";
		if ($params['data']["placeholder"]) $placeholder = 'data-placeholder="'.$params['data']["placeholder"].'"';
		$s  = "\n<select ".$placeholder;
		$s .= self::base($params);
		$s .= isset($params['class'])	? " class=\"form-elmnt select {$params['class']}\""		: ' class="form-elmnt select"';
		$s .= '>';
		$s .= isset($params['prompt'])	? "\n<option value=\"\">{$params['prompt']}</option>" : '';
		if(isset($params['data']) AND is_array($params['data'])) {
			foreach($params['data'] as $k => $v) {
				$class="";
				if($params['dataclass'][$k]!=""){
					$class = 'class="'.$params['dataclass'][$k].'"';
				}
				$style="";
				if($params['datastyle'][$k]!=""){
					$style = 'style="'.$params['datastyle'][$k].'"';
				}
				if(isset($params['selected']) AND $params['selected']==$k) {
					$s .= "\n<option ".$class." ".$style." value=\"{$k}\" selected=\"selected\">{$v}</option>";
				} else if ($k=="placeholder"){
					//placeholder
					$s .= "\n<option></option>";
				} else {
					$s .= "\n<option ".$class." ".$style." value=\"{$k}\">{$v}</option>";
				}
			}
		}
		$s .= "\n</select>";
		$s .= isset($params['extra']) ? $params['extra'] : '';
		return $s;
	}


	# print select
	public static function select2group($params = array()) {
		$placeholder="";
		if (!empty($params['data']["placeholder"])) $placeholder = 'data-placeholder="'.$params['data']["placeholder"].'"';
		$s  = "\n<select ".$placeholder;
		$s .= self::base($params);
		$s .= isset($params['class'])	? " class=\"form-elmnt select {$params['class']}\""		: ' class="form-elmnt select"';
		$s .= '>';
		$s .= isset($params['prompt'])	? "\n<option value=\"\">{$params['prompt']}</option>" : '';
		if(isset($params['data']) AND is_array($params['data'])) {
			foreach($params['data'] as $k => $v) {
				if(is_array($v)) {
					if ($k=="placeholder"){
						//placeholder
						$s .= "\n<option></option>";
					}else{
						$s.= '<optgroup label="'.$k.'">'."\n";
						foreach($v as $c=>$d) {
							$class="";
							if(!empty($params['dataclass'][$k][$c])){
								$class = 'class="'.$params['dataclass'][$k][$c].'"';
							}
							$style="";
							if(!empty($params['datastyle'][$k][$c])){
								$style = 'style="'.$params['datastyle'][$k][$c].'"';
							}
							if(isset($params['selected']) AND $params['selected']==$c) {
								$s .= "\n<option ".$class." ".$style." value=\"{$c}\" selected=\"selected\">{$d}</option>";
							} else if ($c=="placeholder"){
								//placeholder
								$s .= "\n<option></option>";
							} else {
								$s .= "\n<option ".$class." ".$style." value=\"{$c}\">{$d}</option>";
							}
						}
						$s.= '</optgroup>'."\n";
					}
				} else {
					$class="";
					if($params['dataclass'][$k]!=""){
						$class = 'class="'.$params['dataclass'][$k].'"';
					}
					if(isset($params['selected']) AND $params['selected']==$k) {
						$s .= "\n<option ".$class." ".$style." value=\"{$k}\" selected=\"selected\">{$v}</option>";
					} else if ($k=="placeholder"){
						//placeholder
						$s .= "\n<option></option>";
					} else {
						$s .= "\n<option ".$class." ".$style." value=\"{$k}\">{$v}</option>";
					}
				}
			}
		}
		$s .= "\n</select>";
		$s .= isset($params['extra']) ? $params['extra'] : '';
		return $s;
	}

	/*
	 * Méthode qui retourne un ou plusieurs champs de type checkbox
	 * Exemple d'appel:
	 * 		echo plxForm::checkbox(
	 *			array(
	 *				array('id'=>'field1', 'name'=>'cb[]', 'value'=>'x', 'label'=>'label_text_field_1' ),
	 *				array('id'=>'field2', 'name'=>'cb[]', 'value'=>'y', 'label'=>'label_text_field_2' )
	 *			);
	 * @param   array(array(id, name, class, onclick, disabled, style, tabindex, class, value, checked, label , extra))
	 * @return  string
	 * @author	Evrard J.
	 */
	public static function checkbox($params = array()) {
		$s = '';
		if(!empty($params)) {
			$x = 0;
			foreach($params as $k => $v) {
				$v['id'] = isset($v['id'])		? $v['id']											: 'id_chkbx_'.$x++;
				$s .= "\n<input type=\"checkbox\"";
				$s .= self::base($v);
				$s .= isset($v['class'])		? " class=\"form-elmnt checkbox {$v['class']}\""		: ' class="form-elmnt checkbox"';
				$s .= isset($v['value'])		? " value=\"{$v['value']}\""							: '';
				$s .= isset($v['checked'])		? ' checked="checked"'									: '';
				$s .= ' /> ';
				if(isset($v['label']))
					$s .= self::label(array('for'=>$v['id'], 'label'=>$v['label'], 'style'=>'display:inline'));
				$s .= isset($v['extra']) ? $v['extra'] : '';
			}
		}
		return $s;
	}

	/**
	 * Méthode qui retourne un ou plusieurs champs input de type radio
	 * Exemple de paramètres: array(0=>array('id'=>'1', 'name'=>'rd[]', 'value'=>'x', 'label'=>'label_text' ))
	 * @param   array(array(id, name, onclick, disabled, style, tabindex, class, value, label, extra))
	 * @return  string
	 * @author	Evrard J.
	 */
	public static function radio($params = array()) {
		$s = '';
		if(!empty($params)) {
			$x = 0;
			foreach($params as $k => $v) {
				$v['id'] = isset($v['id'])		? $v['id']											: 'id_radio_'.$x++;
				$s .= "\n<input type=\"radio\"";
				$s .= self::base($v);
				$s .= isset($v['class'])		? " class=\"form-elmnt radio {$v['class']}\""		: ' class="form-elmnt radio"';
				$s .= isset($v['value'])		? " value=\"{$v['value']}\""							: '';
				$s .= isset($v['checked'])		? ' checked="checked"'									: '';
				$s .= ' /> ';
				if(isset($v['label']))
					$s .= plxForm::label(array('for'=>$v['id'], 'label'=>$v['label'], 'style'=>'display:inline'));
				$s .= isset($v['extra']) ? $v['extra'] : '';
			}
		}
		return $s;
	}

	/**
	 * Méthode qui retourne un champ input de type button
	 * @param   array(id, name, onclick, disabled, style, tabindex, class, value, extra)
	 * @return  string
	 * @author	Evrard J.
	 */
	public static function button($params = array()) {
		$s = "\n<input type=\"button\"";
		$s .= self::base($params);
		$s .= isset($params['class'])	? " class=\"form-elmnt button {$params['class']}\""	: ' class="form-elmnt button"';
		$s .= isset($params['value'])	? " value=\"{$params['value']}\""					: '';
		$s .= ' />';
		$s .= isset($params['extra']) ? $params['extra'] : '';
		return $s;
	}

	/**
	 * Méthode qui retourn un champ input de type submit
	 * @param   array(id, name, onclick, disabled, tabindex, style, class, value, extra)
	 * @return  string
	 * @author	Evrard J.
	 */
	public static function submit($params = array()) {
		$s  = "\n<input type=\"submit\"";
		$s .= plxForm::base($params);
		$s .= isset($params['class'])	? " class=\"form-elmnt submit {$params['class']}\""	: ' class="form-elmnt submit"';
		$s .= isset($params['value'])	? " value=\"{$params['value']}\""					: '';
		$s .= ' />';
		$s .= isset($params['extra']) ? $params['extra'] : '';
		return $s;
	}

	/**
	 * Méthode qui retourne un champ input de type hidden
	 * @param   array(id, name, value)
	 * @return  string
	 * @author	Evrard J.
	 */
	public static function hidden($params = array()) {
		$s = "\n<input type=\"hidden\"";
		$s .= isset($params['class'])	? " class=\"form-elmnt hidden {$params['class']}\""	: ' class="form-elmnt hidden"';
		if(isset($params['id']))
			$s .= " id=\"{$params['id']}\"";
		else
			$s .= isset($params['name']) 	? " id=\"id_{$params['name']}\"" 	: " id=\"id_".rand(1000,9999)."\"";
		$s .= isset($params['name'])		? " name=\"{$params['name']}\""		: '';
		$s .= isset($params['value'])		? " value=\"{$params['value']}\""	: '';
		$s .= ' />';
		return $s;
	}

	/**
	 * Méthode qui retourne un champ input de type password
	 * @param   array(id, name, onclick, disabled, style, tabindex, class, value, size, maxlength, extra)
	 * @return  string
	 * @author	Evrard J.
	 */
	public static function password($params = array()) {
		$s  = "\n<input type=\"password\"";
		$s .= self::base($params);
		$s .= isset($params['class'])		? " class=\"form-elmnt password {$params['class']}\""	: ' class="form-elmnt password"';
		$s .= isset($params['value'])		? " value=\"{$params['value']}\""						: '';
		$s .= isset($params['size'])		? " size=\"{$params['size']}\""							: '';
		$s .= isset($params['maxlength'])	? " maxlength=\"{$params['maxlength']}\""				: '';
		$s .= " />";
		$s .= isset($params['extra']) ? $params['extra'] : '';
		return $s;
	}

	/**
	 * Méthode qui retourne un champ input de type file
	 * @param   array(id, onclick, disabled, style, tabindex, class, extra)
	 * @return  string
	 * @author	Evrard J.
	 */
	public static function file($params = array()) {
		$s  = "\n<input type=\"file\"";
		$s .= self::base($params);
		$s .= isset($params['class'])		? " class=\"form-elmnt file {$params['class']}\""	: ' class="form-elmnt file"';
		$s .= " />";
		$s .= isset($params['extra']) ? $params['extra'] : '';
		return $s;
	}
}
?>