<?php

if (class_exists('spxvisualwizardWidget')) {

	class plxarchives extends spxvisualwizardWidget {

		private $plxMotor = null; # instance de l'objet plxMotor
		private $aArts = array(); # tableau des articles
		private $excludeCats = array(); # tableau des categories exclues de l'affichage

		/**
		 * Constructeur de la classe
		 *
		 * @param	default_lang	langue par défaut utilisée par PluXml
		 * @return	null
		 * @authors	je-evrard
		 **/
		public function __construct($default_lang) {
			# Appel du constructeur de la classe visualwizardWidget (obligatoire)
			parent::__construct($default_lang);
			$this->name = 'plxarchives';
			$this->title = $this->__( 'title-plxarchives', 'spxvisualwizard' );
			$this->description = $this->__( 'desc-plxarchives', 'spxvisualwizard' );
			$this->category = "Pluxml";




			$options = array(

				'color' => '#000000',
				'level' => 'h3',

				'excludeCats' => '',
				// sort, rsort, alpha, random
				'sort' => 'sort',
				// by_year by_category by_author by_title by_title',
				'sortby' => 'by_title',
				'format' => "#art_date <i class=\"fa fa-user\" aria-hidden=\"true\"></i> #art_author : #art_link",

				'arttitle' => 'Articles',
				'statictitle' => 'Pages',

				'displaystatic' => '1',



			);

			$this->default_options = $this->merge_advanced_default_options($options);

			$this->add_shortcode();
		}

		public function add_shortcode(){
			add_shortcode( 'viw_plxarchives', array( $this, 'shortcode_frontend' ) );
		}

		public function shortcode_frontend( $atts, $content ){
			extract( shortcode_atts( $this->default_options, $atts ) );

			$this->plxMotor = plxMotor::getInstance();

			$format = stripslashes(rawurldecode ($format)) ;
			$format = str_replace("&#34;", '"', $format);

			$class  = 'viw_plxarchives';

			/* add advanced option */
			$oaof = $this->get_o_advanced_options_front( $atts );
			$styleadv = $oaof["style"];
			$idadv = $oaof["id"];
			$class .= $oaof["class"];
			$extra = $oaof["extra"];
			/* end add advanced option */

			$cssstyle = 'style="';
			if ($color !='') $cssstyle .= 'color: '.$color.' ;';
			$cssstyle .= $styleadv;
			$cssstyle .= '"';

			$mycontent = $this->get_content($level, $sort, $format, $sortby, $arttitle, $statictitle, $displaystatic);

			$html = '<div '.$cssstyle.$idadv.' class="'.$class.'" '.$extra.' >'.$mycontent.'</div>';

			return $html;
		}

		public function get_content($level, $sort, $format, $sortby, $arttitle, $statictitle, $displaystatic){

			$this->getArticles($sort, $sortby) ;
			$s = '';

			$s.= '<'.$level.' class="viw_plxarchives-title" >
			'.$arttitle.'
			</'.$level.'>';

			if($this->aArts) {
				$s.= '<div class="viw_plxarchives-artcontainer" >';
				if($sortby=='by_title') {
					$s.= '<ul class="viw_plxarchives-ul" >';
					foreach($this->aArts as $k => $v) {
						$s.= $this->dspList($v, $format);
					}
					$s.= '</ul>';
				} else {
					foreach($this->aArts as $k => $v) {
						$s.= '<p class="viw_plxarchives-p">'.plxUtils::strCheck($k).'</p>';
						$s.= '<ul>';
						$s.= $this->dspList($v, $format);
						$s.= '</ul>';
					}
				}
				$s.= '</div>';
			} else {
				$s.= '<p>No article</p>';
			}
			if ($displaystatic=='1') {
				$s.= '<'.$level.' class="viw_plxarchives-title" >
				'.$statictitle.'
				</'.$level.'>';

				$s.=  $this->getStatics() ;
			}
			return $s;


		}

		/**
		 * Méthode qui récupère les articles en fonction des critères d'affichage et de tri
		 *
		 * @author	Stephane F
		 **/
		public function getArticles($sort='sort', $sortby='by_year') {

			$array=array();
			$plxGlob_arts = clone $this->plxMotor->plxGlob_arts;

			if($files = $plxGlob_arts->query('/^[0-9]{4}.[home|'.$this->plxMotor->activeCats.',]*.[0-9]{3}.[0-9]{12}.[a-z0-9-]+.xml$/','art',$sort,0,false,'before')) {
				foreach($files as $filename) {
					if(preg_match('/[0-9]{4}.((?:[0-9]|home|,)*(?:'.$this->plxMotor->activeCats.'|home)(?:[0-9]|home|,)*).[0-9]{3}.[0-9]{12}.[a-z0-9-]+.xml$/',$filename,$capture)){
						$catIds=explode(',',$capture[1]);
						foreach($catIds as $catId) {
							if(!in_array($catId, $this->excludeCats)) {
								$tmp = $this->plxMotor->parseArticle(PLX_ROOT.$this->plxMotor->aConf['racine_articles'].$filename);

								if($catId=='000')
									$catName = L_UNCLASSIFIED;
								elseif($catId=='home')
									$catName = L_HOMEPAGE;
								else
									$catName = $this->plxMotor->aCats[$catId]['name'];

								$art = array(
									'id'		=> $tmp['numero'],
									'category'	=> $catName,
									'url' 		=> $this->plxMotor->urlRewrite('?article'.intval($tmp['numero']).'/'.$tmp['url']),
									'title' 	=> $tmp['title'],
									'year'		=> substr($tmp['date'], 0,4),
									'date'		=> $tmp['date'],
									'author'	=> $this->plxMotor->aUsers[$tmp['author']]['name'],
								);
								$array[] = $art;
							}
						}
					}
				}
			}
			if($array) {
				# tri multi-dimentionnel
				foreach ($array as $key => $row) {
					switch ($sortby) {
						case 'by_year':
							$a1[$key] = $row['year'];
							break;
						case 'by_category':
							$a1[$key] = $row['category'];
							break;
						case 'by_author':
							$a1[$key] = $row['author'];
							break;
						case 'by_title':
							$a1[$key] = $row['title'];
							break;
					}
					$a2[$key] = $row['date'];
				}

				array_multisort($a1, ($sortby=='by_year'?SORT_DESC:SORT_ASC), $a2, ($sort=='asc'?SORT_ASC:SORT_DESC), $array);

				# préparation pour l'affichage
				foreach ($array as $key => $row) {
					switch ($sortby) {
						case 'by_year':
							$this->aArts[$row['year']][$row['id']] = $row;
							break;
						case 'by_category':
							$this->aArts[$row['category']][$row['id']] = $row;
							break;
						case 'by_author':
							$this->aArts[$row['author']][$row['id']] = $row;
							break;
						case 'by_title':
							$this->aArts[$row['title']][$row['id']] = $row;
							break;
					}
				}
			}

		}

		private function dspList($v, $format) {
			$s='';
			foreach($v as $art) {
				$row = str_replace('#art_date', plxDate::formatDate($art['date'],'#num_day/#num_month/#num_year(4)'), $format);
				$row = str_replace('#art_link', '<a href="'.$art['url'].'">'.plxUtils::strCheck($art['title']).'</a>', $row);
				$row = str_replace('#art_author', plxUtils::strCheck($art['author']), $row);
				$s.= '<li><i class="fa fa-angle-right" aria-hidden="true"></i> '.$row.'</li>';
			}
			return $s;
		}

		public function getStatics() {

			$s.='<ul class="viw_plxarchives-ul" >';
				# Les pages statiques
				foreach($this->plxMotor->aStats as $stat_num => $stat_info) {
					if($stat_info['active']==1 AND $stat_num!=$plxMotor->aConf['homestatic']) {
						$s.='<li><a href="'.$this->plxMotor->urlRewrite("?static".$stat_num."/".plxUtils::strCheck($stat_info["url"])).'" ><i class="fa fa-angle-right" aria-hidden="true"></i> '.$stat_info["name"].'</a></li>';
					}

				}
			$s.='</ul>';

			return $s;
		}



		public function shortcode_options( $atts ){
			extract( shortcode_atts( $this->default_options, $atts ) );

			$options = array(

				array(
					'title' => $this->__( 'Design', 'spxvisualwizard' ),
					'type' => 'grouptitle',
				),

				array(
					'param_name' => 'color',
					'title' => $this->__( 'Color', 'spxvisualwizard' ),
					'desc' => $this->__( 'The color of the text.', 'spxvisualwizard' ),
					'type' => 'colordefault',
					'selected' => $color,
				),

				array(
					'param_name' => 'level',
					'title' => $this->__( 'Level', 'spxvisualwizard' ),
					'desc' => $this->__( 'The level of the title', 'spxvisualwizard' ),
					'type' => 'select',
					'selected' => $level,
					'data' => array(
						'h1' =>  $this->__( 'h1', 'spxvisualwizard' ),
						'h2' =>  $this->__( 'h2', 'spxvisualwizard' ),
						'h3' =>  $this->__( 'h3', 'spxvisualwizard' ),
						'h4' =>  $this->__( 'h4', 'spxvisualwizard' ),
						'h5' =>  $this->__( 'h5', 'spxvisualwizard' ),
						'h6' =>  $this->__( 'h6', 'spxvisualwizard' )



					),
				),

				array(
					'title' => $this->__( 'Articles', 'spxvisualwizard' ),
					'type' => 'grouptitle',
				),

				array(
					'param_name' => 'arttitle',
					'title' => $this->__( 'Title', 'spxvisualwizard' ),
					'desc' => '',
					'type' => 'text',
					'value' => $arttitle,
				),

				array(
					'param_name' => 'format',
					'title' => $this->__( 'Format', 'spxvisualwizard' ),
					'desc' => '#art_date &lt;i class=&quot;fa fa-user&quot; aria-hidden=&quot;true&quot;&gt;&lt;/i&gt; #art_author : #art_link',
					'type' => 'textarea',
					'value' => $format,
				),





				array(
					'param_name' => 'sortby',
					'title' => $this->__( 'Sort by', 'spxvisualwizard' ),
					'desc' => '',
					'type' => 'select',
					'selected' => $sortby,
					'data' => array(
						'by_title' =>  $this->__( 'By title', 'spxvisualwizard' ),
						'by_author' =>  $this->__( 'By author', 'spxvisualwizard' ),
						'by_category' =>  $this->__( 'By category', 'spxvisualwizard' ),
						'by_year' =>  $this->__( 'By year', 'spxvisualwizard' ),

					),
				),



				array(
					'param_name' => 'sort',
					'title' => $this->__( 'Sort method', 'spxvisualwizard' ),
					'desc' => '',
					'type' => 'select',
					'selected' => $sort,
					'data' => array(
						'sort' =>  $this->__( 'sort', 'spxvisualwizard' ),
						'rsort' =>  $this->__( 'rsort', 'spxvisualwizard' ),
						'alpha' =>  $this->__( 'alpha', 'spxvisualwizard' ),
						'random' =>  $this->__( 'random', 'spxvisualwizard' ),

					),
				),

				array(
					'title' => $this->__( 'Static', 'spxvisualwizard' ),
					'type' => 'grouptitle',
				),

				array(
					'param_name' => 'displaystatic',
					'title' => $this->__( 'Display static', 'spxvisualwizard' ),
					'desc' => '',
					'type' => 'select',
					'selected' => $displaystatic,
					'data' => array(
						'1' =>  $this->__( 'Yes', 'spxvisualwizard' ),
						'0' =>  $this->__( 'No', 'spxvisualwizard' ),





					),
				),

				array(
					'param_name' => 'statictitle',
					'title' => $this->__( 'Title', 'spxvisualwizard' ),
					'desc' => '',
					'type' => 'text',
					'value' => $statictitle,
				),


			);


			return $this->get_options($options);
		}


		public function template_wireframe() {
			?>
				<script id="id_viw_template_widget_wireframe_plxarchives" type="x-tmpl-mustache">
				<div class="viw_ui-widget-title-container">
					<div class="viw_ui-widget-icon-container">
						<span class="viw_ui-widget-icon viw_ui-widget-icon-{{ type }}"></span>
					</div>
					{{ type | uppercase }}
				</div>
				</script>
			<?php
		}

	}

}
?>